#!/usr/bin/perl

# Wcal installer 2.1
# by Joel Savignon

use strict;

sub cancel
{
    my $code = shift;
    print "\n**********************************\n";
    printf "*** Installation canceled (%02d) ***\n", $code;
    print "**********************************\n\n";
    exit $code;
}
if (scalar(@ARGV) != 7 )
{
    print "---------------------------------------------------------\n";
    print "Don't run this script directly, instead do \"make install\"\n";
    cancel 1;
}

# Test perl version
eval("require 5.004");
if ($@)
{
    print "---------------------------------------------------------\n";
    print "Wcal requires perl version 5.004 or above.\n";
    cancel 2;
}

my $interactive = shift;
my $default_prefix = shift;
my $httpd_group_guesses = shift;
my $default_wcal_user = shift;
my $default_html_dir = shift;
my $default_conf_dir = shift;
my $default_data_dir = shift;

# Test perl modules
eval("use Date::Manip");
if ($@)
{
    print "---------------------------------------------------------\n";
    print "Wcal requires module Date::Manip.\n";
    print "This comes with Wcal, please install it.\n";
    cancel 3;
}
eval("use Data::Dumper");
if ($@)
{
    print "---------------------------------------------------------\n";
    print "Wcal requires module Data::Dumper.\n";
    print "This comes with Wcal, please install it.\n";
    cancel 4;
}


my $wcal_user = $default_wcal_user;
if ($interactive)
{
    print "User to run wcal as [$default_wcal_user]: ";
    my $wuser = <STDIN>;
    chomp $wuser;
    if ($wuser)
    {
	$wcal_user = $wuser;
    }
}

my $wcal_uid = getpwnam($wcal_user);
if ($wcal_uid)
{
    print "User $wcal_user found : id = $wcal_uid\n";
}
else
{
    print "---------------------------------------------------------\n";
    print "Can't find a suitable user to run wcald as. You should\n";
    print "create one, called \"$wcal_user\". On Linux and some other\n";
    print "Unices this is done by issuing as root the command\n";
    print "\"useradd $wcal_user\".\n";
    cancel 5;
}

my $wcal_gid;
my $wcal_group;
my $default_wcal_group = "nobody";
foreach $wcal_group (split(/\s+/,$httpd_group_guesses))
{
    $wcal_gid = getgrnam($wcal_group);
    if ($wcal_gid)
    {
	$default_wcal_group = $wcal_group;
	last;
    }
}
$wcal_group = $default_wcal_group;
if ($interactive)
{
    print "Group to run wcal as [$default_wcal_group]: ";
    my $wgroup = <STDIN>;
    chomp $wgroup;
    if ($wgroup)
    {
	$wcal_group = $wgroup;
    }
}

$wcal_gid = getgrnam($wcal_group);
if (defined $wcal_gid)
{
    print "Group $wcal_group found : id = $wcal_gid\n";
}
else
{
    print "---------------------------------------------------------\n";
    print "Can't find a suitable group to run wcald as. You should\n";
    print "create one, called \"$wcal_group\". On Linux and some other\n";
    print "Unices this is done by issuing as root the command\n";
    print "\"groupadd $wcal_group\".\n";
    cancel 6;
}

my $html_dir = $default_html_dir;
if ($interactive)
{
    print "HTML directory for Wcal [$default_html_dir]: ";
    my $whtml = <STDIN>;
    chomp $whtml;
    if ($whtml)
    {
	$html_dir = $whtml;
    }
}

if ($html_dir !~ /^\//)
{
    print "---------------------------------------------------------\n";
    print "The directory path must be absolute.\n";
    cancel 7;
}

my $conf_dir = $default_conf_dir;
if ($interactive)
{
    print "Configuration directory for Wcal [$default_conf_dir]: ";
    my $wconf = <STDIN>;
    chomp $wconf;
    if ($wconf)
    {
	$conf_dir = $wconf;
    }
}

my $data_dir = $default_data_dir;
if ($interactive)
{
    print "Database directory for Wcal [$default_data_dir]: ";
    my $wdatad = <STDIN>;
    chomp $wdatad;
    if ($wdatad)
    {
	$data_dir = $wdatad;
    }
}

my $prefix = $default_prefix;
if ($interactive)
{
    print "Prefix for for Wcal [$default_prefix]: ";
    my $wpref = <STDIN>;
    chomp $wpref;
    if ($wpref)
    {
	$prefix = $wpref;
    }
}


my $rdata = "";
my $i;
srand(time ^ $$);
for ($i = 0; $i < 30; $i++)
{
    $rdata .= sprintf("%c",int(rand(94))+33);
}
if ($interactive)
{
    print "Password for Wcal (at least 30 characters) [$rdata]: ";
    my $wdata = <STDIN>;
    chomp $wdata;
    if ($wdata)
    {
	$rdata = $wdata;
    }
}

# Create wcal.conf from wcal.conf.in
open(FILE1, "wcal.conf.in");
open(FILE2, ">wcal.conf");
my $line;
while ($line = <FILE1>)
{
    $line =~ s/_X_PASSWORD/password = $rdata;/g;
    $line =~ s/_X_INSTALL_DIR/install dir = $html_dir;/g;
    print FILE2 $line;
}
close(FILE1);
close(FILE2);

# Create wcald from wcald.in
open(FILE1, "wcald.in");
open(FILE2, ">wcald");
my $line;
while ($line = <FILE1>)
{
    $line =~ s/_X_DIRECTORY/$html_dir/g;
    $line =~ s/_X_CONF/$conf_dir/g;
    $line =~ s/_X_DATA/$data_dir/g;
    print FILE2 $line;
}
close(FILE1);
close(FILE2);

# Create HTML directories
if (!-d $html_dir)
{
    if (!mkdir($html_dir,0750))
    {
	print "---------------------------------------------------------\n";
	print "Can't create HTML directory $html_dir\n";
	cancel 8;
    }
}
if (!-d "$html_dir/images")
{
    if (!mkdir("$html_dir/images",0750))
    {
	print "---------------------------------------------------------\n";
	print "Can't create image directory $html_dir/images\n";
	cancel 9;
    }
}
if (!-d "$html_dir/doc")
{
    if (!mkdir("$html_dir/doc",0750))
    {
	print "---------------------------------------------------------\n";
	print "Can't create image directory $html_dir/doc\n";
	cancel 10;
    }
}

# Create database directory
if (!-d $data_dir)
{
    if (!mkdir($data_dir,0750))
    {
	print "---------------------------------------------------------\n";
	print "Can't create database directory $data_dir\n";
	cancel 11;
    }
}
chown($wcal_uid, $wcal_gid, $data_dir);
chmod(0700,$data_dir);

# Create config directory
if (!-d $conf_dir)
{
    if (!mkdir($conf_dir,0755))
    {
	print "---------------------------------------------------------\n";
	print "Can't create config directory $conf_dir\n";
	cancel 12;
    }
}

# Install wcald
system("cp wcald $prefix/bin/wcald");
chmod(0755,"$prefix/bin/wcald");
      
# Install wcal.conf
system("cp wcal.conf $conf_dir/wcal.conf");
chmod(0640,"$conf_dir/wcal.conf");
chown($wcal_uid, $wcal_gid, "$conf_dir/wcal.conf");

# Install wcal.msg
system("cp wcal.msg $conf_dir/wcal.msg");
chmod(0640,"$conf_dir/wcal.msg");
chown($wcal_uid, $wcal_gid, "$conf_dir/wcal.msg");

# Install wcal.def
system("cp wcal.def $conf_dir/wcal.def");
chmod(0640,"$conf_dir/wcal.def");
chown($wcal_uid, $wcal_gid, "$conf_dir/wcal.def");

# Install wcal.sdays
system("cp wcal.sdays $conf_dir/wcal.sdays");
chmod(0640,"$conf_dir/wcal.sdays");
chown($wcal_uid, $wcal_gid, "$conf_dir/wcal.sdays");

# install img and doc files
system("cp images/* $html_dir/images");
system("cp -r doc/* $html_dir/doc");
system("chown -R $wcal_user $html_dir");
system("chgrp -R $wcal_group $html_dir");
system("chmod -R 0750 $html_dir");

open(CMD, "uname -s|");
my $os = <CMD>;
close(CMD);
chomp($os);
my $carg = "";
if ($os eq "SunOS")
{
    $carg = "-lsocket -lnsl";
}

# Compile index.cgi binary
open(FILE1, "index.c.in");
open(FILE2, ">index.c");
my $line;
while ($line = <FILE1>)
{
    $line =~ s/_X_CONF/$conf_dir/g;
    print FILE2 $line;
}
close(FILE1);
close(FILE2);
print "Compiling index.c\n";
if (system("gcc $carg -Wall -O2 index.c -o index.cgi"))
{
    print "---------------------------------------------------------\n";
    print "Error while compiling index.c\n";
    cancel 13;
}

system("strip index.cgi");

# Compile checkalarm binary
open(FILE1, "checkalarm.c.in");
open(FILE2, ">checkalarm.c");
my $line;
while ($line = <FILE1>)
{
    $line =~ s/_X_CONF/$conf_dir/g;
    print FILE2 $line;
}
close(FILE1);
close(FILE2);

print "Compiling checkalarm.c\n";
if (system("gcc $carg -Wall -O2 checkalarm.c -o checkalarm"))
{
    print "---------------------------------------------------------\n";
    print "Error while compiling index.c\n";
    cancel 14;
}
system("strip checkalarm");

# Install checkalarm binary
system("cp checkalarm $prefix/sbin/checkalarm");
chmod(0755,"$prefix/sbin/checkalarm");

# install cgi files
system("cp index.cgi $html_dir");
chown($wcal_uid, $wcal_gid, "$html_dir/index.cgi");
chmod(0750,"$html_dir/index.cgi");

system("ln -sf $html_dir/index.cgi $html_dir/fusion.cgi");

print ("\nWcal successfully installed\n");
print ("Now log as $wcal_user user and run $prefix/bin/wcald &\n");
print ("Wait a few seconds and then try http://your.machine/wcal\n");
